@@ -21,7 +21,6 @@ import cairo
 import math
 import os
 from socket import gethostname
-import wimpiggy.lowlevel
 from wimpiggy.lowlevel import (
                const,                                       #@UnresolvedImport
                add_event_receiver,                          #@UnresolvedImport
@@ -37,6 +36,8 @@ from wimpiggy.lowlevel import (
                XRemoveFromSaveSet,                          #@UnresolvedImport
                XSetInputFocus,                              #@UnresolvedImport
                XKillClient,                                 #@UnresolvedImport
+               send_wm_take_focus,                          #@UnresolvedImport
+               send_wm_delete_window,                       #@UnresolvedImport
                sendConfigureNotify,                         #@UnresolvedImport
                configureAndNotify,                          #@UnresolvedImport
                substructureRedirect                         #@UnresolvedImport
@@ -236,6 +237,7 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
 
         log("new window %s", hex(client_window.xid))
 
+        self._composite = None
         self.client_window = client_window
         self._internal_set_property("client-window", client_window)
         add_event_receiver(client_window, self)
@@ -280,8 +282,9 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
 
     def do_unmanaged(self, wm_exiting):
         remove_event_receiver(self.client_window, self)
-        self._composite.disconnect(self._damage_forward_handle)
-        self._composite.destroy()
+        if self._composite:
+            self._composite.disconnect(self._damage_forward_handle)
+            self._composite.destroy()
 
 gobject.type_register(BaseWindowModel)
 
@@ -962,13 +965,18 @@ class WindowModel(BaseWindowModel):
         # XSetInputFocus well, while Qt apps ignore (!!!) WM_TAKE_FOCUS
         # (unless they have a modal window), and just expect to get focus from
         # the WM's XSetInputFocus.
+        success = True
         if self._input_field:
             log("... using XSetInputFocus")
-            trap.swallow(XSetInputFocus, self.client_window, now)
+            if not trap.swallow(XSetInputFocus, self.client_window, now):
+                log("XSetInputFocus failed...")
+                success = False
         if "WM_TAKE_FOCUS" in self.get_property("protocols"):
             log("... using WM_TAKE_FOCUS")
-            trap.swallow(wimpiggy.lowlevel.send_wm_take_focus,
-                         self.client_window, now)
+            if not trap.swallow(send_wm_take_focus, self.client_window, now):
+                log("WM_TAKE_FOCUS failed...")
+                success = False
+        return success
 
     ################################
     # Killing clients:
@@ -976,8 +984,7 @@ class WindowModel(BaseWindowModel):
 
     def request_close(self):
         if "WM_DELETE_WINDOW" in self.get_property("protocols"):
-            trap.swallow(wimpiggy.lowlevel.send_wm_delete_window,
-                         self.client_window)
+            trap.swallow(send_wm_delete_window, self.client_window)
         else:
             # You don't wanna play ball?  Then no more Mr. Nice Guy!
             self.force_quit()
@@ -1089,7 +1096,7 @@ class WindowView(gtk.Widget):
         x2i = int(math.ceil(x2))
         y2i = int(math.ceil(y2))
         transformed = gtk.gdk.Rectangle(x1i, y1i, x2i - x1i, y2i - y1i)
-#        print ("damage (%s, %s, %s, %s) -> expose on (%s, %s, %s, %s)" %
+#        print("damage (%s, %s, %s, %s) -> expose on (%s, %s, %s, %s)" %
 #               (event.area.x, event.area.y, event.area.width, event.area.height,
 #                transformed.x, transformed.y, transformed.width, transformed.height))
         self._image_window.invalidate_rect(transformed, False)
@@ -1101,7 +1108,7 @@ class WindowView(gtk.Widget):
         debug = False
 
         if debug:
-            print ("redrawing rectangle at (%s, %s, %s, %s)"
+            print("redrawing rectangle at (%s, %s, %s, %s)"
                    % (event.area.x, event.area.y,
                       event.area.width, event.area.height))
 
