@@ -10,6 +10,8 @@ import gobject
 import cairo
 import re
 import os
+import time
+from collections import deque
 
 from wimpiggy.util import (n_arg_signal,
                            gtk_main_quit_really,
@@ -22,6 +24,7 @@ from xpra.protocol import Protocol
 from xpra.keys import mask_to_names, MODIFIER_NAMES
 from xpra.platform.gui import ClientExtras
 from xpra.scripts.main import ENCODINGS
+from xpra.version_util import is_compatible_with
 
 import xpra
 default_capabilities = {"__prerelease_version": xpra.__version__}
@@ -85,9 +88,10 @@ class ClientWindow(gtk.Window):
         self._metadata = {}
         self._override_redirect = override_redirect
         self._new_backing(w, h)
-        self._failed_pixbuf_index = 0
         self._refresh_timer = None
         self._refresh_requested = False
+        # used for only sending focus events *after* the window is mapped:
+        self._been_mapped = False
 
         self.update_metadata(metadata)
 
@@ -217,11 +221,7 @@ class ClientWindow(gtk.Window):
             loader.close()
             pixbuf = loader.get_pixbuf()
             if not pixbuf:
-                if self._failed_pixbuf_index<10:
-                    log.error("failed %s pixbuf=%s data len=%s" % (coding, pixbuf, len(img_data)))
-                elif self._failed_pixbuf_index==10:
-                    log.error("too many pixbuf failures! (will no longer be logged)")
-                    self._failed_pixbuf_index += 1
+                log.error("failed %s pixbuf=%s data len=%s" % (coding, pixbuf, len(img_data)))
             else:
                 self._backing.draw_pixbuf(gc, pixbuf, 0, 0, x, y, width, height)
         self.window.invalidate_rect(gtk.gdk.Rectangle(x, y, width, height), False)
@@ -258,6 +258,8 @@ class ClientWindow(gtk.Window):
             self._client.send(["map-window", self._id, x, y, w, h])
             self._pos = (x, y)
             self._size = (w, h)
+        self._been_mapped = True
+        gobject.idle_add(self._focus_change)
 
     def do_configure_event(self, event):
         log("Got configure event")
@@ -277,7 +279,16 @@ class ClientWindow(gtk.Window):
         self.window.move_resize(x, y, w, h)
         self._new_backing(w, h)
 
+    def destroy(self):
+        self._unfocus()
+        gtk.Window.destroy(self)
+
+    def _unfocus(self):
+        if self._client._focused==self._id:
+            self._client.update_focus(self._id, False)
+
     def do_unmap_event(self, event):
+        self._unfocus()
         if not self._override_redirect:
             self._client.send(["unmap-window", self._id])
 
@@ -331,9 +342,9 @@ class ClientWindow(gtk.Window):
         self._button_action(scroll_map[event.direction], event, False)
 
     def _focus_change(self, *args):
-        log.debug("_focus_change(%s)" % str(args))
-        self._client.update_focus(self._id,
-                                  self.get_property("has-toplevel-focus"))
+        log("_focus_change(%s)", args)
+        if self._been_mapped:
+            self._client.update_focus(self._id, self.get_property("has-toplevel-focus"))
 
 gobject.type_register(ClientWindow)
 
@@ -346,6 +357,7 @@ class XpraClient(gobject.GObject):
 
     def __init__(self, conn, opts):
         gobject.GObject.__init__(self)
+        self.start_time = time.time()
         self._window_to_id = {}
         self._id_to_window = {}
         title = opts.title
@@ -365,16 +377,29 @@ class XpraClient(gobject.GObject):
             self.jpegquality = 50
 
         self.server_capabilities = {}
+
+        self.can_ping = False
+        self.mmap_enabled = False
+        self.server_start_time = -1
+        self.server_platform = ""
+        self.server_actual_desktop_size = None
+        self.server_desktop_size = None
+        self.server_randr = False
+        self.pixel_counter = deque(maxlen=100)
+        self.server_latency = deque(maxlen=100)
+        self.server_load = None
+        self.client_latency = deque(maxlen=100)
         self.bell_enabled = True
         self.notifications_enabled = True
-        self._client_extras = ClientExtras(self, opts)
-        self.clipboard_enabled = opts.clipboard and self._client_extras.supports_clipboard()
         self.send_damage_sequence = False
-        self.mmap_enabled = False
-        self.supports_mmap = opts.mmap and self._client_extras.supports_mmap()
+        self.clipboard_enabled = False
         self.mmap = None
         self.mmap_file = None
         self.mmap_size = 0
+
+        self._client_extras = ClientExtras(self, opts)
+        self.clipboard_enabled = opts.clipboard and self._client_extras.supports_clipboard()
+        self.supports_mmap = opts.mmap and self._client_extras.supports_mmap()
         if self.supports_mmap:
             try:
                 import mmap
@@ -404,9 +429,11 @@ class XpraClient(gobject.GObject):
         self._protocol = Protocol(conn, self.process_packet)
         ClientSource(self._protocol)
 
+        self.keyboard_sync = opts.keyboard_sync
+        self.key_repeat_modifiers = False
         self.key_repeat_delay = -1
         self.key_repeat_interval = -1
-        self.keycodes_pressed = {}
+        self.keys_pressed = {}
         self._raw_keycodes_feature = False
         self._focus_modifiers_feature = False
         self._remote_version = None
@@ -424,21 +451,14 @@ class XpraClient(gobject.GObject):
             bw = (self._protocol._recv_counter / 1024) * 1000/ delay;
             self._protocol._recv_counter = 0;
             log.debug("Bandwidth is ", bw, "kB/s, max ", self.max_bandwidth, "kB/s")
-
             q = self.jpegquality
             if bw > self.max_bandwidth:
-                q -= 10;
+                q -= 10
             elif bw < self.max_bandwidth:
-                q += 5;
-
-            if q > 95:
-                q = 95;
-            elif q < 10:
-                q = 10;
-
+                q += 5
+            q = max(10, min(95 ,q))
             self.send_jpeg_quality(q)
             return True
-
         if (self.max_bandwidth):
             gobject.timeout_add(2000, compute_receive_bandwidth, 2000);
 
@@ -448,14 +468,18 @@ class XpraClient(gobject.GObject):
         gtk.main()
 
     def cleanup(self):
-        self._client_extras.exit()
+        if self._client_extras:
+            self._client_extras.exit()
+            self._client_extras = None
         if self._protocol:
             self._protocol.close()
+            self._protocol = None
         self.clean_mmap()
 
     def clean_mmap(self):
         if self.mmap_file and os.path.exists(self.mmap_file):
             os.unlink(self.mmap_file)
+            self.mmap_file = None
 
     def quit(self, *args):
         gtk_main_quit_really()
@@ -531,63 +555,91 @@ class XpraClient(gobject.GObject):
         id = self._window_to_id[window]
         if not self._raw_keycodes_feature:
             """ versions before 0.0.7.24 only accept 4 parameters (no keyval, keycode, ...)
-                also used on win32 and osx since those don't have keymaps/keycode
+                also used on win32 and osx since those don't have valid keymaps/keycode (yet?)
             """
             # Apparently some weird keys (e.g. "media keys") can have no keyval or
             # no keyval name (I believe that both give us a None here).
             # Another reason to use the _raw_keycodes_feature wherever possible.
-            if name is not None:
-                self.send(["key-action", id, name, depressed, modifiers])
-            return
-        keycode = event.hardware_keycode
-        log.debug("key_action(%s,%s,%s) modifiers=%s, name=%s, state=%s, keyval=%s, string=%s, keycode=%s" % (event, window, depressed, modifiers, name, event.state, event.keyval, event.string, keycode))
-        self.send(["key-action", id, nn(name), depressed, modifiers, nn(event.keyval), nn(event.string), nn(keycode)])
-        if keycode and self.key_repeat_delay>0 and self.key_repeat_interval>0:
-            self._key_repeat(depressed, keycode)
+            if name is None:
+                return
+            self.send(["key-action", id, name, depressed, modifiers])
+            keyval = ""
+            keycode = 0
+        else:
+            keyval = nn(event.keyval)
+            keycode = event.hardware_keycode
+            log.debug("key_action(%s,%s,%s) modifiers=%s, name=%s, state=%s, keyval=%s, string=%s, keycode=%s" % (event, window, depressed, modifiers, name, event.state, event.keyval, event.string, keycode))
+            self.send(["key-action", id, nn(name), depressed, modifiers, keyval, nn(event.string), nn(keycode)])
+        if self.keyboard_sync and self.key_repeat_delay>0 and self.key_repeat_interval>0:
+            self._key_repeat(id, depressed, name, keyval, keycode)
 
-    def _key_repeat(self, depressed, keycode):
-        if not depressed and keycode in self.keycodes_pressed:
+    def _key_repeat(self, id, depressed, name, keyval, keycode):
+        """ this method takes care of scheduling the sending of
+            "key-repeat" packets to the server so that it can
+            maintain a consistent keyboard state.
+        """
+        #we keep track of which keys are still pressed in a dict,
+        #the key is either the keycode (if _raw_keycodes_feature) or the key name (otherwise)
+        if keycode==0:
+            if not self.key_repeat_modifiers:
+                #we can't handle key-repeat by key name without this feature
+                return
+            key = name
+        else:
+            key = keycode
+        if not depressed and key in self.keys_pressed:
             """ stop the timer and clear this keycode: """
-            log.debug("key repeat: clearing timer for %s", keycode)
-            gobject.source_remove(self.keycodes_pressed[keycode])
-            del self.keycodes_pressed[keycode]
-        elif depressed and keycode not in self.keycodes_pressed:
+            log.debug("key repeat: clearing timer for %s / %s", name, keycode)
+            gobject.source_remove(self.keys_pressed[key])
+            del self.keys_pressed[key]
+        elif depressed and key not in self.keys_pressed:
             """ we must ping the server regularly for as long as the key is still pressed: """
+            #TODO: we can have latency measurements (see ping).. use them?
             LATENCY_JITTER = 100
             MIN_DELAY = 20
             delay = max(self.key_repeat_delay-LATENCY_JITTER, MIN_DELAY)
             interval = max(self.key_repeat_interval-LATENCY_JITTER, MIN_DELAY)
+            def send_key_repeat():
+                if self.key_repeat_modifiers:
+                    #supports extended mode, send the extra data:
+                    (_, _, current_mask) = gtk.gdk.get_default_root_window().get_pointer()
+                    modifiers = self.mask_to_names(current_mask)
+                    packet = ["key-repeat", id, name, keyval, keycode, modifiers]
+                else:
+                    packet = ["key-repeat", keycode]
+                self.send_now(packet)
             def continue_key_repeat(*args):
                 #if the key is still pressed (redundant check?)
                 #confirm it and continue, otherwise stop
-                log.debug("continue_key_repeat")
-                if keycode in self.keycodes_pressed:
-                    self.send_now(["key-repeat", keycode])
+                log.debug("continue_key_repeat for %s / %s", name, keycode)
+                if key in self.keys_pressed:
+                    send_key_repeat()
                     return  True
                 else:
-                    del self.keycodes_pressed[keycode]
+                    del self.keys_pressed[key]
                     return  False
             def start_key_repeat(*args):
                 #if the key is still pressed (redundant check?)
                 #confirm it and start repeat:
-                log.debug("start_key_repeat")
-                if keycode in self.keycodes_pressed:
-                    self.send_now(["key-repeat", keycode])
-                    self.keycodes_pressed[keycode] = gobject.timeout_add(interval, continue_key_repeat)
+                log.debug("start_key_repeat for %s / %s", name, keycode)
+                if key in self.keys_pressed:
+                    send_key_repeat()
+                    self.keys_pressed[key] = gobject.timeout_add(interval, continue_key_repeat)
                 else:
-                    del self.keycodes_pressed[keycode]
+                    del self.keys_pressed[key]
                 return  False   #never run this timer again
-            log.debug("key repeat: starting timer for %s with delay %s and interval %s", keycode, delay, interval)
-            self.keycodes_pressed[keycode] = gobject.timeout_add(delay, start_key_repeat)
+            log.debug("key repeat: starting timer for %s / %s with delay %s and interval %s", name, keycode, delay, interval)
+            self.keys_pressed[key] = gobject.timeout_add(delay, start_key_repeat)
 
     def clear_repeat(self):
-        for timer in self.keycodes_pressed.values():
+        for timer in self.keys_pressed.values():
             gobject.source_remove(timer)
-        self.keycodes_pressed = {}
+        self.keys_pressed = {}
 
     def query_xkbmap(self):
         self.xkbmap_layout, self.xkbmap_variant, self.xkbmap_variants = self._client_extras.get_layout_spec()
         self.xkbmap_print, self.xkbmap_query, self.xmodmap_data = self._client_extras.get_keymap_spec()
+        self.xkbmap_mod_clear, self.xkbmap_mod_add = self._client_extras.get_keymap_modifiers()
 
     def _keys_changed(self, *args):
         self._keymap = gtk.gdk.keymap_get_default()
@@ -622,7 +674,7 @@ class XpraClient(gobject.GObject):
                 self.send(["focus", _id, self.mask_to_names(current_mask)])
             else:
                 self.send(["focus", _id])
-        log.debug("update_focus(%s,%s) _focused=%s", id, gotit, self._focused)
+        log("update_focus(%s,%s) _focused=%s", id, gotit, self._focused)
         if gotit and self._focused is not id:
             self.clear_repeat()
             send_focus(id)
@@ -669,6 +721,10 @@ class XpraClient(gobject.GObject):
             capabilities_request["xkbmap_query"] = self.xkbmap_query
         if self.xmodmap_data:
             capabilities_request["xmodmap_data"] = self.xmodmap_data
+        if self.xkbmap_mod_clear:
+            capabilities_request["xkbmap_mod_clear"] = self.xkbmap_mod_clear
+        if self.xkbmap_mod_add:
+            capabilities_request["xkbmap_mod_add"] = self.xkbmap_mod_add
         capabilities_request["cursors"] = True
         capabilities_request["bell"] = True
         capabilities_request["clipboard"] = self.clipboard_enabled
@@ -682,13 +738,42 @@ class XpraClient(gobject.GObject):
         capabilities_request["desktop_size"] = [root_w, root_h]
         capabilities_request["png_window_icons"] = True
         capabilities_request["damage_sequence"] = True
+        capabilities_request["ping"] = True
         key_repeat = self._client_extras.get_keyboard_repeat()
         if key_repeat:
-            capabilities_request["key_repeat"] = key_repeat
+            delay_ms,interval_ms = key_repeat
+            capabilities_request["key_repeat"] = (delay_ms,interval_ms)
+        capabilities_request["keyboard_sync"] = self.keyboard_sync and key_repeat
         if self.mmap_file:
             capabilities_request["mmap_file"] = self.mmap_file
         self.send(["hello", capabilities_request])
 
+    def send_ping(self):
+        if self.can_ping:
+            self.send(["ping", long(1000*time.time())])
+
+    def _process_ping_echo(self, packet):
+        (_, echoedtime, l1, l2, l3, cl) = packet[:6]
+        diff = long(1000*time.time()-echoedtime)
+        self.server_latency.append(diff)
+        self.server_load = (l1, l2, l3)
+        if cl>=0:
+            self.client_latency.append(cl)
+        log("ping echo server load=%s, measured client latency=%s", self.server_load, cl)
+
+    def _process_ping(self, packet):
+        assert self.can_ping
+        (_, echotime) = packet[:2]
+        try:
+            (fl1, fl2, fl3) = os.getloadavg()
+            l1,l2,l3 = long(fl1*1000), long(fl2*1000), long(fl3*1000)
+        except:
+            l1,l2,l3 = 0,0,0
+        sl = -1
+        if len(self.server_latency)>0:
+            sl = self.server_latency[-1]
+        self.send(["ping_echo", echotime, l1, l2, l3, sl])
+
     def send_jpeg_quality(self, q):
         assert q>0 and q<100
         self.jpegquality = q
@@ -719,14 +804,6 @@ class XpraClient(gobject.GObject):
         hash = hmac.HMAC(password, salt)
         self.send_hello(hash.hexdigest())
 
-    def version_no_minor(self, version):
-        if not version:
-            return version
-        p = version.rfind(".")
-        if p>0:
-            return version[:p]
-        return version
-
     def _process_hello(self, packet):
         (_, capabilities) = packet
         self.server_capabilities = capabilities
@@ -734,18 +811,18 @@ class XpraClient(gobject.GObject):
             self.session_name = capabilities.get("session_name", "Xpra")
         import glib
         glib.set_application_name(self.session_name)
-        self._raw_keycodes_feature = capabilities.get("raw_keycodes_feature", False) and \
-                            (self.xkbmap_print is not None or self.xkbmap_query is not None or self.xmodmap_data is not None)
+        self._raw_keycodes_feature = capabilities.get("raw_keycodes_feature", False) and self._client_extras.supports_raw_keycodes()
         self._focus_modifiers_feature = capabilities.get("raw_keycodes_feature", False)
         if "deflate" in capabilities:
             self._protocol.enable_deflate(capabilities["deflate"])
         self._remote_version = capabilities.get("__prerelease_version")
-        if self.version_no_minor(self._remote_version) != self.version_no_minor(xpra.__version__):
-            log.error("sorry, I only know how to talk to v%s.x servers", self.version_no_minor(xpra.__version__))
+        if not is_compatible_with(self._remote_version):
             self.quit()
             return
-        if "desktop_size" in capabilities:
-            avail_w, avail_h = capabilities["desktop_size"]
+        self.server_actual_desktop_size = capabilities.get("actual_desktop_size")
+        self.server_desktop_size = capabilities.get("desktop_size")
+        if self.server_desktop_size:
+            avail_w, avail_h = self.server_desktop_size
             root_w, root_h = gtk.gdk.get_default_root_window().get_size()
             if (avail_w, avail_h) < (root_w, root_h):
                 log.warn("Server's virtual screen is too small -- "
@@ -755,9 +832,9 @@ class XpraClient(gobject.GObject):
                          "http://xpra.org/trac/ticket/10"
                          % (avail_w, avail_h, root_w, root_h))
         self._protocol._send_size = capabilities.get("packet_size", False)
-        randr = capabilities.get("resize_screen", False)
-        log.debug("server has randr: %s" % randr)
-        if randr:
+        self.server_randr = capabilities.get("resize_screen", False)
+        log.debug("server has randr: %s", self.server_randr)
+        if self.server_randr:
             display = gtk.gdk.display_get_default()
             i=0
             while i<display.get_n_screens():
@@ -770,17 +847,22 @@ class XpraClient(gobject.GObject):
             self.encoding = e
         self.bell_enabled = capabilities.get("bell", False)
         self.notifications_enabled = capabilities.get("notifications", False)
-        clipboard_server_support = capabilities.get("clipboard", True) 
+        clipboard_server_support = capabilities.get("clipboard", True)
         self.clipboard_enabled = clipboard_server_support and self._client_extras.supports_clipboard()
         self.send_damage_sequence = capabilities.get("damage_sequence", False)
+        self.can_ping = capabilities.get("ping", False)
         self.mmap_enabled = self.supports_mmap and self.mmap_file and capabilities.get("mmap_enabled")
         if self.mmap_enabled:
             log.info("mmap enabled using %s", self.mmap_file)
+        self.server_start_time = capabilities.get("start_time", -1)
+        self.server_platform = capabilities.get("platform")
+
         #the server will have a handle on the mmap file by now, safe to delete:
         self.clean_mmap()
         #ui may want to know this is now set:
         self.emit("clipboard-toggled")
         self.key_repeat_delay, self.key_repeat_interval = capabilities.get("key_repeat", (-1,-1))
+        self.key_repeat_modifiers = capabilities.get("key_repeat_modifiers", False)
         self.emit("handshake-complete")
         if clipboard_server_support:
             #from now on, we will send a message to the server whenever the clipboard flag changes:
@@ -820,8 +902,11 @@ class XpraClient(gobject.GObject):
             packet_sequence = packet[8]
         else:
             packet_sequence = None
-        window = self._id_to_window[id]
+        window = self._id_to_window.get(id)
+        if not window:
+            return      #window is already gone!
         window.draw(x, y, width, height, coding, data)
+        self.pixel_counter.append((time.time(), width*height))
         if packet_sequence and self.send_damage_sequence:
             self.send_now(["damage-sequence", packet_sequence])
 
@@ -918,6 +1003,8 @@ class XpraClient(gobject.GObject):
         "bell": _process_bell,
         "notify_show": _process_notify_show,
         "notify_close": _process_notify_close,
+        "ping": _process_ping,
+        "ping_echo": _process_ping_echo,
         "window-metadata": _process_window_metadata,
         "configure-override-redirect": _process_configure_override_redirect,
         "lost-window": _process_lost_window,
