@@ -73,13 +73,15 @@ class ClientExtras(ClientExtrasBase):
             quit_item.hide()
 
             self.macapp.insert_app_menu_item(self.make_aboutmenuitem(), 0)
-            self.macapp.insert_app_menu_item(self.make_bellmenuitem(), 1)
-            self.macapp.insert_app_menu_item(self.make_notificationsmenuitem(), 2)
-            self.macapp.insert_app_menu_item(self.make_encodingsmenuitem(), 3)
-            self.macapp.insert_app_menu_item(self.make_layoutsmenuitem(), 4)
-            self.macapp.insert_app_menu_item(self.make_jpegsubmenu(), 5)
-            self.macapp.insert_app_menu_item(self.make_refreshmenuitem(), 6)
-            self.macapp.insert_app_menu_item(gtk.SeparatorMenuItem(), 7)
+            self.macapp.insert_app_menu_item(self.make_sessioninfomenuitem(), 1)
+            self.macapp.insert_app_menu_item(self.make_bellmenuitem(), 2)
+            self.macapp.insert_app_menu_item(self.make_notificationsmenuitem(), 3)
+            self.macapp.insert_app_menu_item(self.make_encodingsmenuitem(), 4)
+            self.macapp.insert_app_menu_item(self.make_layoutsmenuitem(), 5)
+            self.macapp.insert_app_menu_item(self.make_jpegsubmenu(), 6)
+            self.macapp.insert_app_menu_item(self.make_refreshmenuitem(), 7)
+            self.macapp.insert_app_menu_item(self.make_raisewindowsmenuitem(), 8)
+            self.macapp.insert_app_menu_item(gtk.SeparatorMenuItem(), 9)
 
             #dock menu (does not work!)
             #menu = gtk.Menu()
@@ -123,9 +125,9 @@ class ClientExtras(ClientExtrasBase):
         Carbon.Snd.SysBeep(1)
 
     def grok_modifier_map(self, display_source):
-        map = ClientExtrasBase.grok_modifier_map(self, display_source)
-        map["meta"] = 1 << 3
-        return  map
+        modifiers = ClientExtrasBase.grok_modifier_map(self, display_source)
+        modifiers["meta"] = 1 << 3
+        return  modifiers
 
     def get_data_dir(self):
         return  os.environ.get("XDG_DATA_DIRS", os.getcwd())
