@@ -14,8 +14,12 @@ pygtk.require("2.0")
 import gtk
 import gobject
 import webbrowser
+import time
+import datetime
 
+from xpra.platform import XPRA_LOCAL_SERVERS_SUPPORTED
 from xpra.scripts.main import ENCODINGS
+from xpra.keys import XMODMAP_MOD_DEFAULTS, XMODMAP_MOD_ADD, XMODMAP_MOD_CLEAR
 from wimpiggy.util import gtk_main_quit_really
 from wimpiggy.log import Logger
 log = Logger()
@@ -80,6 +84,10 @@ class ClientExtrasBase(object):
     def __init__(self, client, opts):
         self.client = client
         self.license_text = None
+        self.session_info_window = None
+        self.about_dialog = None
+        self.tray_icon = opts.tray_icon
+        self.session_name = opts.session_name
         self.set_window_icon(opts.window_icon)
 
     def set_window_icon(self, window_icon):
@@ -96,7 +104,11 @@ class ClientExtrasBase(object):
         gtk_main_quit_really()
 
     def exit(self):
-        pass
+        self.close_about()
+        self.close_session_info()
+
+    def supports_mmap(self):
+        return XPRA_LOCAL_SERVERS_SUPPORTED
 
     def supports_clipboard(self):
         return self.clipboard_helper is not None
@@ -139,15 +151,25 @@ class ClientExtrasBase(object):
         """ layout, variant, variants"""
         return None,None,None
 
+    def supports_raw_keycodes(self):
+        return False
+
+    def get_keymap_modifiers(self):
+        return  XMODMAP_MOD_CLEAR, XMODMAP_MOD_ADD
+
     def get_keymap_spec(self):
         """ xkbmap_print, xkbmap_query, xmodmap """
-        return None,None,None
+        # default map for non xposix clients:
+        return None,None,XMODMAP_MOD_DEFAULTS
 
     def get_keyboard_repeat(self):
         """ (delay_ms,interval_ms) or None"""
         return None
 
     def about(self, *args):
+        if self.about_dialog:
+            self.about_dialog.present()
+            return
         dialog = gtk.AboutDialog()
         def on_website_hook(dialog, web, *args):
             webbrowser.open("http://xpra.org/")
@@ -170,13 +192,187 @@ class ClientExtrasBase(object):
         if pixbuf:
             dialog.set_logo(pixbuf)
         dialog.set_program_name("Xpra")
-        def response(*args):
-            dialog.destroy()
-            self.about_dialog = None
-        dialog.connect("response", response)
+        comments = ''
+        try:
+            from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, REVISION, LOCAL_MODIFICATIONS
+            comments += "Built on %s by %s. %s" % (BUILT_ON, BUILT_BY, BUILD_DATE)
+            if LOCAL_MODIFICATIONS==0:
+                comments += "\n(svn revision %s)" % REVISION
+            else:
+                comments += "\n(svn revision %s with %s local changes)" % (REVISION, LOCAL_MODIFICATIONS)
+        except Exception, e:
+            log.error("could not find the build information: %s", e)
+        dialog.set_comments(comments)
+        dialog.connect("response", self.close_about)
         self.about_dialog = dialog
         dialog.show()
 
+    def close_about(self, *args):
+        try:
+            if self.about_dialog:
+                self.about_dialog.destroy()
+                self.about_dialog = None
+        except:
+            log.error("closing about dialog", exc_info=True)
+
+
+    def session_info(self, *args):
+        if self.session_info_window:
+            self.session_info_window.present()
+            return
+        window = gtk.Window()
+        window.set_title(self.session_name or "Session Info")
+        window.set_destroy_with_parent(True)
+        window.set_resizable(True)
+        window.set_decorated(True)
+        pixbuf = None
+        if self.tray_icon:
+            pixbuf = self.get_pixbuf(self.tray_icon)
+        if not pixbuf:
+            pixbuf = self.get_pixbuf("xpra.png")
+        if pixbuf:
+            window.set_icon(pixbuf)
+        window.set_position(gtk.WIN_POS_CENTER)
+
+        # Contents box
+        vbox = gtk.VBox(False, 0)
+        vbox.set_spacing(2)
+        table = gtk.Table(1, columns=2)
+        table.set_col_spacings(3)
+        table.set_row_spacings(3)
+        vbox.add(table)
+        def add_row(row, label, widget):
+            l_al = gtk.Alignment(xalign=1.0, yalign=0.5, xscale=0.0, yscale=0.0)
+            l_al.add(label)
+            table.attach(l_al, 0, 1, row, row + 1, xpadding=10)
+            w_al = gtk.Alignment(xalign=0.0, yalign=0.5, xscale=0.0, yscale=0.0)
+            w_al.add(widget)
+            table.attach(w_al, 1, 2, row, row + 1, xpadding=10)
+            return row + 1
+
+        # now add some rows with info:
+        row = 0
+        self.server_version_label = gtk.Label()
+        row = add_row(row, gtk.Label("Server Version"), self.server_version_label)
+        if self.client.server_platform:
+            row = add_row(row, gtk.Label("Server Platform"), gtk.Label(self.client.server_platform))
+        self.server_randr_label = gtk.Label()
+        row = add_row(row, gtk.Label("Server RandR Support"), self.server_randr_label)
+        if self.client.can_ping:
+            self.server_load_label = gtk.Label()
+            row = add_row(row, gtk.Label("Server Load"), self.server_load_label)
+            self.server_latency_label = gtk.Label()
+            row = add_row(row, gtk.Label("Server Latency"), self.server_latency_label)
+            self.client_latency_label = gtk.Label()
+            row = add_row(row, gtk.Label("Client Latency"), self.client_latency_label)
+        if self.client.server_start_time>0:
+            self.session_started_label = gtk.Label()
+            row = add_row(row, gtk.Label("Session Started"), self.session_started_label)
+        self.session_connected_label = gtk.Label()
+        row = add_row(row, gtk.Label("Session Connected"), self.session_connected_label)
+        self.windows_managed_label = gtk.Label()
+        row = add_row(row, gtk.Label("Windows Managed"), self.windows_managed_label)
+        self.pixels_per_second_label = gtk.Label()
+        row = add_row(row, gtk.Label("Pixels/s"), self.pixels_per_second_label)
+
+        def populate_table(*args):
+            if not self.session_info_window:
+                return False
+            self.client.send_ping()
+            def settimedeltastr(label, from_time):
+                delta = datetime.timedelta(seconds=(long(time.time())-long(from_time)))
+                label.set_text(str(delta))
+            v = self.client._remote_version or "unknown"
+            if self.client.mmap_enabled:
+                self.server_version_label.set_text("%s (mmap in use)" % v)
+            else:
+                self.server_version_label.set_text(v)
+            size_info = ""
+            if self.client.server_actual_desktop_size:
+                w,h = self.client.server_actual_desktop_size
+                size_info = " - %sx%s" % (w,h)
+            if self.client.server_randr:
+                self.server_randr_label.set_text("Yes%s" % size_info)
+            else:
+                self.server_randr_label.set_text("No%s" % size_info)
+            if self.client.can_ping:
+                if self.client.server_load:
+                    self.server_load_label.set_text("  ".join([str(x/1000.0) for x in self.client.server_load]))
+                if len(self.client.server_latency)>0:
+                    avg = sum(self.client.server_latency)/len(self.client.server_latency)
+                    self.server_latency_label.set_text("%sms  (%sms)" % (self.client.server_latency[-1], avg))
+                if len(self.client.client_latency)>0:
+                    avg = sum(self.client.client_latency)/len(self.client.client_latency)
+                    self.client_latency_label.set_text("%sms  (%sms)" % (self.client.client_latency[-1], avg))
+            if self.client.server_start_time>0:
+                settimedeltastr(self.session_started_label, self.client.server_start_time)
+            settimedeltastr(self.session_connected_label, self.client.start_time)
+            real, redirect = 0, 0
+            for w in self.client._window_to_id.keys():
+                if w._override_redirect:
+                    redirect +=1
+                else:
+                    real += 1
+            self.windows_managed_label.set_text("%s  (%s transient)" % (real, redirect))
+            pixels = "n/a"
+            if len(self.client.pixel_counter)>0:
+                total = 0
+                now = time.time()
+                mint = now-20       #ignore records older than 20 seconds
+                startt = now        #when we actually start counting from
+                for (t, count) in self.client.pixel_counter:
+                    if t>=mint:
+                        total += count
+                        startt = min(t, startt)
+                if total>0 and startt!=now:
+                    def pixelstr(v):
+                        if v>1000*1000*1000:
+                            return "%sG" % (long(v/1000/1000/100)/10.0)
+                        elif v>1000*1000:
+                            return "%sM" % (long(v/1000/100)/10.0)
+                        elif v>1000:
+                            return "%sK" % (long(v/100)/10.0)
+                        else:
+                            return str(v)
+                    t, last = self.client.pixel_counter[-1]
+                    if t<now-5:
+                        last = 0
+                    avg = long(total/(now-startt))
+                    pixels = "%s  (%s)" % (pixelstr(last), pixelstr(avg))
+
+            self.pixels_per_second_label.set_text(pixels)
+            return True
+        gobject.timeout_add(1000, populate_table)
+
+        window.set_border_width(15)
+        window.add(vbox)
+        window.set_geometry_hints(vbox)
+        def window_deleted(*args):
+            self.session_info_window = None
+        window.connect('delete_event', window_deleted)
+        self.add_close_accel(window, self.close_session_info)
+        self.session_info_window = window
+        populate_table()
+        window.show_all()
+
+    def close_session_info(self, *args):
+        try:
+            if self.session_info_window:
+                self.session_info_window.destroy()
+                self.session_info_window = None
+        except:
+            log.error("closing session info", exc_info=True)
+
+    def add_close_accel(self, window, callback):
+        # key accelerators
+        accel_group = gtk.AccelGroup()
+        accel_group.connect_group(ord('w'), gtk.gdk.CONTROL_MASK, gtk.ACCEL_LOCKED, callback)
+        window.add_accel_group(accel_group)
+        accel_group = gtk.AccelGroup()
+        escape_key, modifier = gtk.accelerator_parse('Escape')
+        accel_group.connect_group(escape_key, modifier, gtk.ACCEL_LOCKED |  gtk.ACCEL_VISIBLE, callback)
+        window.add_accel_group(accel_group)
+
 
 
     def grok_modifier_map(self, display_source):
@@ -295,7 +491,10 @@ class ClientExtrasBase(object):
         self.menu_shown = True
 
     def make_aboutmenuitem(self):
-        return  self.menuitem("About", "information.png", None, self.about)
+        return  self.menuitem("About Xpra", "information.png", None, self.about)
+
+    def make_sessioninfomenuitem(self):
+        return  self.menuitem(self.session_name or "Session Info", "statistics.png", None, self.session_info)
 
     def make_bellmenuitem(self):
         def bell_toggled(*args):
@@ -332,7 +531,7 @@ class ClientExtrasBase(object):
         return self.clipboard_menuitem
 
     def make_encodingsmenuitem(self):
-        encodings = self.menuitem("Encoding", "configure.png", "Choose picture data encoding", None)
+        encodings = self.menuitem("Encoding", "encoding.png", "Choose picture data encoding", None)
         self.encodings_submenu = gtk.Menu()
         encodings.set_submenu(self.encodings_submenu)
         self.popup_menu_workaround(self.encodings_submenu)
@@ -346,7 +545,7 @@ class ClientExtrasBase(object):
                     if self.client.encoding!=enc:
                         self.client.set_encoding(enc)
                         log.debug("setting encoding to %s", enc)
-                        self.update_jpeg_menu()
+                        self.set_jpegmenu()
                         self.updated_menus()
                 encoding_item.set_active(encoding==self.client.encoding)
                 encoding_item.set_sensitive(encoding in self.client.server_capabilities.get("encodings", ["rgb24"]))
@@ -444,13 +643,14 @@ class ClientExtrasBase(object):
             qi.connect('activate', set_jpeg_quality)
             self.jpeg_submenu.append(qi)
         self.jpeg_submenu.show_all()
-        def set_jpegmenu(*args):
-            if self.jpeg_quality:
-                self.jpeg_quality.set_sensitive("jpeg"==self.client.encoding)
-                self.updated_menus()
-        self.client.connect("handshake-complete", set_jpegmenu)
+        self.client.connect("handshake-complete", self.set_jpegmenu)
         return self.jpeg_quality
 
+    def set_jpegmenu(self, *args):
+        if self.jpeg_quality:
+            self.jpeg_quality.set_sensitive("jpeg"==self.client.encoding)
+            self.updated_menus()
+
     def updated_menus(self):
         """ subclasses may override this method - see darwin """
         pass
@@ -461,6 +661,13 @@ class ClientExtrasBase(object):
             self.client.send_refresh_all()
         return self.menuitem("Refresh", "retry.png", None, force_refresh)
 
+    def make_raisewindowsmenuitem(self):
+        def raise_windows(*args):
+            for win in self.client._window_to_id.keys():
+                if not win._override_redirect:
+                    win.present()
+        return self.menuitem("Raise Windows", "raise.png", None, raise_windows)
+
     def make_disconnectmenuitem(self):
         return self.menuitem("Disconnect", "quit.png", None, self.quit)
 
@@ -477,6 +684,7 @@ class ClientExtrasBase(object):
         self.client.connect("handshake-complete", set_menu_title)
 
         menu.append(self.make_aboutmenuitem())
+        menu.append(self.make_sessioninfomenuitem())
         menu.append(gtk.SeparatorMenuItem())
         menu.append(self.make_bellmenuitem())
         menu.append(self.make_notificationsmenuitem())
@@ -492,6 +700,7 @@ class ClientExtrasBase(object):
             self.jpeg_submenu = None
         menu.append(self.make_layoutsmenuitem())
         menu.append(self.make_refreshmenuitem())
+        menu.append(self.make_raisewindowsmenuitem())
         #menu.append(item("Options", "configure", None, self.options))
         menu.append(gtk.SeparatorMenuItem())
         menu.append(self.make_disconnectmenuitem())
