@@ -12,6 +12,7 @@ import time
 from optparse import OptionParser
 import logging
 from subprocess import Popen, PIPE
+import signal
 
 import xpra
 from xpra.bencode import bencode
@@ -98,6 +99,9 @@ def main(script_file, cmdline):
     parser.add_option("--no-pulseaudio", action="store_false",
                       dest="pulseaudio", default=True,
                       help="Disable pulseaudio support via X11 root window properties")
+    parser.add_option("--no-keyboard-sync", action="store_false",
+                      dest="keyboard_sync", default=True,
+                      help="Disable keyboard state synchronization, prevents keys from repeating on high latency links but also may disrupt applications which access the keyboard directly")
     parser.add_option("--password-file", action="store",
                       dest="password_file", default=None,
                       help="The file containing the password required to connect (useful to secure TCP mode)")
@@ -171,8 +175,10 @@ def main(script_file, cmdline):
     if options.encoding and options.encoding not in ENCODINGS:
         parser.error("illegal encoding")
 
-    logging.root.setLevel(logging.INFO)
-    if options.debug is not None:
+    def toggle_logging(level):
+        if not options.debug:
+            logging.root.setLevel(level)
+            return
         categories = options.debug.split(",")
         for cat in categories:
             if cat.startswith("-"):
@@ -181,8 +187,20 @@ def main(script_file, cmdline):
                 logger = logging.root
             else:
                 logger = logging.getLogger(cat)
-            logger.setLevel(logging.DEBUG)
+            logger.setLevel(level)
+    def toggle_debug_on(*args):
+        toggle_logging(logging.DEBUG)
+    def toggle_debug_off(*args):
+        toggle_logging(logging.INFO)
+        
+    if options.debug is not None:
+        toggle_debug_on()
+    else:
+        toggle_debug_off()
     logging.root.addHandler(logging.StreamHandler(sys.stderr))
+    if os.name=="posix":
+        signal.signal(signal.SIGUSR1, toggle_debug_on)
+        signal.signal(signal.SIGUSR2, toggle_debug_off)
 
     mode = args.pop(0)
 
@@ -327,7 +345,6 @@ def run_client(parser, opts, extra_args, detach):
             sys.stdout.write("handshake-complete: detaching")
             app.quit()
         app.connect("handshake-complete", do_detach)
-    import signal
     signal.signal(signal.SIGINT, app.quit)
     try:
         app.run()
@@ -360,19 +377,19 @@ def run_stop(parser, opts, extra_args):
             else:
                 break
         if final_state is DotXpra.DEAD:
-            print "xpra at %s has exited." % display_desc["display"]
+            print("xpra at %s has exited." % display_desc["display"])
             sys.exit(0)
         elif final_state is DotXpra.UNKNOWN:
-            print ("How odd... I'm not sure what's going on with xpra at %s"
+            print("How odd... I'm not sure what's going on with xpra at %s"
                    % display_desc["display"])
             sys.exit(1)
         elif final_state is DotXpra.LIVE:
-            print "Failed to shutdown xpra at %s" % display_desc["display"]
+            print("Failed to shutdown xpra at %s" % display_desc["display"])
             sys.exit(1)
         else:
             assert False
     else:
-        print "Sent shutdown command"
+        print("Sent shutdown command")
 
 def run_list(parser, opts, extra_args):
     assert "gtk" not in sys.modules
