@@ -32,10 +32,13 @@ class ClientExtras(ClientExtrasBase):
         self.setup_tray(opts.tray_icon)
         self.setup_xprops(opts.pulseaudio)
         self.setup_x11_bell()
-        self.setup_pynotify()
+        self.has_dbusnotify = False
+        self.has_pynotify = False
+        self.setup_dbusnotify() or self.setup_pynotify()
         self.setup_clipboard_helper(ClipboardProtocolHelper)
 
     def exit(self):
+        ClientExtrasBase.exit(self)
         if self.tray_widget:
             self.hide_tray()
             self.tray_widget = None
@@ -45,6 +48,11 @@ class ClientExtras(ClientExtrasBase):
         options = ["/usr/share/xpra", "/usr/local/share/xpra"]
         if sys.executable.startswith("/usr/local"):
             options.reverse()
+        try:
+            # test for a local install (from do-build):
+            options.insert(0, os.path.join(os.path.dirname(sys.argv[0]), "..", "share", "xpra"))
+        except:
+            pass
         for x in options:
             if os.path.exists(x):
                 return x
@@ -150,15 +158,31 @@ class ClientExtras(ClientExtrasBase):
         if not self.setup_statusicon(tray_icon_filename):
             log.error("failed to setup system-tray")
 
+    def setup_dbusnotify(self):
+        self.dbus_id = os.environ.get("DBUS_SESSION_BUS_ADDRESS", "")
+        try:
+            import dbus.glib
+            assert dbus.glib
+            bus = dbus.SessionBus()
+            FD_NOTIFICATIONS = 'org.freedesktop.Notifications'
+            obj = bus.get_object(FD_NOTIFICATIONS, '/org/freedesktop/Notifications')
+            self.dbusnotify = dbus.Interface(obj, FD_NOTIFICATIONS)
+            self.has_dbusnotify = True
+            log.info("using dbusnotify: %s(%s)", type(self.dbusnotify), FD_NOTIFICATIONS)
+        except Exception, e:
+            log.error("cannot import pynotify wrapper (turning notifications off) : %s", e)
+        return self.has_dbusnotify
+
     def setup_pynotify(self):
         self.dbus_id = os.environ.get("DBUS_SESSION_BUS_ADDRESS", "")
-        self.has_pynotify = False
         try:
             import pynotify
             pynotify.init("Xpra")
             self.has_pynotify = True
+            log.info("using pynotify: %s", pynotify)
         except ImportError, e:
             log.error("cannot import pynotify wrapper (turning notifications off) : %s", e)
+        return self.has_pynotify
 
     def setup_x11_bell(self):
         self.has_x11_bell = False
@@ -206,49 +230,90 @@ class ClientExtras(ClientExtrasBase):
         device_bell(window, device, bell_class, bell_id, percent, bell_name)
 
     def can_notify(self):
-        return  self.has_pynotify
+        return  self.has_dbusnotify or self.has_pynotify
 
     def show_notify(self, dbus_id, id, app_name, replaces_id, app_icon, summary, body, expire_timeout):
         if self.dbus_id==dbus_id:
             log.error("remote dbus instance is the same as our local one, "
                       "cannot forward notification to ourself as this would create a loop")
             return
-        import pynotify
-        n = pynotify.Notification(summary, body)
-        n.set_urgency(pynotify.URGENCY_LOW)
-        n.set_timeout(expire_timeout)
-        n.show()
+        if self.has_dbusnotify:
+            def cbReply(*args):
+                log("notification reply: %s", args)
+                return False
+            def cbError(*args):
+                log.error("notification error: %s", args)
+                return False
+            try:
+                self.dbusnotify.Notify("Xpra", 0, app_icon, summary, body, [], [], expire_timeout,
+                     reply_handler = cbReply,
+                     error_handler = cbError)
+            except:
+                log.error("dbus notify failed", exc_info=True)
+        elif self.has_pynotify:
+            try:
+                import pynotify
+                n = pynotify.Notification(summary, body)
+                n.set_urgency(pynotify.URGENCY_LOW)
+                n.set_timeout(expire_timeout)
+                n.show()
+            except:
+                log.error("pynotify failed", exc_info=True)
+        else:
+            log.error("notification cannot be displayed, no backend support!")
 
     def close_notify(self, id):
         pass
 
+    def exec_get_keyboard_data(self, cmd):
+        # Find the client's current keymap so we can send it to the server:
+        try:
+            import subprocess
+            process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
+            (out,_) = process.communicate(None)
+            if process.returncode==0:
+                return out
+            log.error("'%s' failed with exit code %s", cmd, process.returncode)
+        except Exception, e:
+            log.error("error running '%s': %s", cmd, e)
+        return None
+
+    def get_keymap_modifiers(self):
+        xmodmap_pm = self.exec_get_keyboard_data(["xmodmap", "-pm"])
+        if not xmodmap_pm:
+            return ClientExtrasBase.get_keymap_modifiers(self)
+        #parse it so we can feed it back to xmodmap (ala "xmodmap -pke")
+        clear = []
+        add = []
+        for line in xmodmap_pm.splitlines()[1:]:
+            if not line:
+                continue
+            parts = line.split()
+            #ie: ['shift', 'Shift_L', '(0x32),', 'Shift_R', '(0x3e)']
+            clear.append("clear %s" % parts[0])
+            if len(parts)>1:
+                nohex = set([x for x in parts[1:] if not x.startswith("(")])
+                add.append("add %s = %s" % (parts[0], " ".join(nohex)))
+        log("get_keymap_modifiers parsed to clear=%s, add=%s", clear, add)
+        return  clear, add
+
+    def supports_raw_keycodes(self):
+        return True
+
     def get_keymap_spec(self):
-        def get_keyboard_data(command, arg):
-            # Find the client's current keymap so we can send it to the server:
-            try:
-                import subprocess
-                cmd = [command, arg]
-                process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
-                (out,_) = process.communicate(None)
-                if process.returncode==0:
-                    return out
-                log.error("'%s %s' failed with exit code %s\n" % (command, arg, process.returncode))
-            except Exception, e:
-                log.error("error running '%s %s': %s\n" % (command, arg, e))
-            return None
-        xkbmap_print = get_keyboard_data("setxkbmap", "-print")
+        xkbmap_print = self.exec_get_keyboard_data(["setxkbmap", "-print"])
         if xkbmap_print is None:
             log.error("your keyboard mapping will probably be incorrect unless you are using a 'us' layout");
-        xkbmap_query = get_keyboard_data("setxkbmap", "-query")
+        xkbmap_query = self.exec_get_keyboard_data(["setxkbmap", "-query"])
         if xkbmap_query is None and xkbmap_print is not None:
             log.error("the server will try to guess your keyboard mapping, which works reasonably well in most cases");
             log.error("however, upgrading 'setxkbmap' to a version that supports the '-query' parameter is preferred");
-        xmodmap_data = get_keyboard_data("xmodmap", "-pke");
+        xmodmap_data = self.exec_get_keyboard_data(["xmodmap", "-pke"]);
         return xkbmap_print, xkbmap_query, xmodmap_data
 
     def get_keyboard_repeat(self):
         try:
-            from wimpiggy.lowlevel import get_key_repeat_rate
+            from wimpiggy.lowlevel import get_key_repeat_rate   #@UnresolvedImport
             delay, interval = get_key_repeat_rate()
             return delay,interval
         except Exception, e:
